<?php

namespace App\Controller;

use App\Controller;
use App\Connexion;
use App\Controller\UserController;

use AltoRouter;
use App\Controller\ErrorController;
use App\Gateway\FormResultGateway;
use App\Gateway\TacticInfoGateway;
use App\Http\HttpCodes;
use App\Http\HttpResponse;
use App\Http\JsonHttpResponse;
use App\Http\ViewHttpResponse;
use App\Model\TacticModel;
use App\Validation\ValidationFail;
use Exception;
use Twig\Loader\FilesystemLoader;


class FrontController{

    private AltoRouter $router;
    private Connexion $con;
    private array $dictControllerRole;

    public function __construct(string $basePath) {
        $this->con = new Connexion(get_database());;
        $this->router = $this->createRouter($basePath);
        $this->dictControllerRole = [
            "UserController" => "public",
            "EditorController" => "public"
        ];
    }

    /**
     * Main behavior of the FrontController
     *
     * @return void
     */
    public function run() : void {
        $this->initializeRouterMap();

        $match = $this->router->match(); 
        if ($match != null){
            $this->controlRoute($match);
        } else {
            $this->diplayViewByKind(ViewHttpResponse::twig("error.html.twig", [], HttpCodes::NOT_FOUND));
        }
        // $this->controlRoute($match["target"]);
        // $this->handleByResponseType($this->matchRoute());
    }

    /**
     * Create a new instance of an AltoRouter
     *
     * @param string $basePath
     * @return AltoRouter
     */
    public function createRouter(string $basePath) : AltoRouter {
        $router = new AltoRouter();
        $router->setBasePath($basePath);
        return $router;
    }

    /**
     * Initialize project's routes
     *
     * @return void
     */
    private function initializeRouterMap() : void {
        $this->router->map("GET", "/", "UserController");
        $this->router->map("GET", "/[a:action]?", "UserController");
        $this->router->map("GET", "/tactic/[a:action]/[i:idTactic]?", "EditorController");
        // $this->router->map("GET", "/tactic/[i:id]/edit", "EditorController");
        // $this->router->map("GET", "/", fn() => (new UserController())->home());
        // $this->router->map("GET", "/tactic/new", fn() => (new EditorController(new TacticModel(new TacticInfoGateway($this->con))))->makeNew());
        // $this->router->map("GET", "/tactic/[i:id]/edit", fn(int $id) => (new EditorController(new TacticModel(new TacticInfoGateway($this->con))))->openEditorFor($id));
    }


    /**
     * Initialize router's settings
     *
     * @return ViewHttpResponse
     */
    private function controlRoute($match){
        $tag = $match['target'];

        $action = $this->getAction($match);
        $this->handleResponseByType($this->tryToCall($tag, $action, $match["params"]));
    }

    // private function sanitizeParam($params) {
    //     foreach ($key, $value : )
    // }

    private function tryToCall($controller, $action, array $params){
        unset($params["action"]);
        $controller = $this->initControllerByRole($controller);
        try {
            if (is_callable(array($controller, $action))){
                return call_user_func_array(array($controller, $action), $params);
            } else {
                return ViewHttpResponse::twig("error.html.twig", [], HttpCodes::NOT_FOUND);
            }
        }
        catch (Exception $e) {
            return ViewHttpResponse::twig("error.html.twig", [], HttpCodes::NOT_FOUND);
        }
    }
    
    private function getAction($match) : string {
        if (isset($match["params"]["action"])){
            return $match["params"]["action"];
        }
        return "home";
    }

    private function initControllerByRole(string $controller) {
        
        $index = $controller;
        $namespace = "\\App\\Controller\\";
        $controller = $namespace.$controller;

        
        if (isset($_SESSION['role'])){
            if ($_SESSION['role'] == $this->dictControllerRole[$index]){
                $controller = new $controller();
                return $controller;
            }
        }
        // A décommenter quand méthode de connexion disponible

        // $connected = (new UserController())->login($this->dictControllerRole[$controller]);
        // if (!$connected){
        //     return "null";
        // }

        $_SESSION['role'] = 'public'; // Remplacer par appel de la méthode de connexion
        $controller = new $controller();

        return $controller;
    }

    /**
     * Redirect the return of the response by the response's type
     *
     * @param array $match
     * @return void
     */
    private function handleResponseByType(HttpResponse $response) : void {
        // $response = call_user_func_array($match['target'], $match['params']);
        http_response_code($response->getCode());
        if ($response instanceof ViewHttpResponse) {

            $this->diplayViewByKind($response);

        } else if ($response instanceof JsonHttpResponse) {
            header('Content-type: application/json');
            echo $response->getJson();
        }
    }

    /**
     * Use the right method to display the response
     *
     * @param ViewHttpResponse $response
     * @return void
     */
    private function diplayViewByKind(ViewHttpResponse $response) : void {
        $file = $response->getFile();
        $args = $response->getArguments();

        switch ($response->getViewKind()) {
            case ViewHttpResponse::REACT_VIEW:
                send_react_front($file, $args);
                break;
            case ViewHttpResponse::TWIG_VIEW:
                try {
                    $loader = new FilesystemLoader('../src/Views/');
                    $twig = new \Twig\Environment($loader);
                    $twig->display($file, $args);
                } catch (\Twig\Error\RuntimeError | \Twig\Error\SyntaxError $e) {
                    http_response_code(500);
                    echo "There was an error rendering your view, please refer to an administrator.\nlogs date: " . date("YYYD, d M Y H:i:s");
                    throw $e;
                }
            break;
        }
    }
}